
import os
import matplotlib
matplotlib.use("QtAgg")
import matplotlib.pyplot as plt

plt.rcParams['text.usetex'] = True
plt.rcParams['text.latex.preamble'] = r"\usepackage{amsmath, amssymb}"
plt.rcParams['pgf.texsystem'] = "xelatex"

from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from PyQt6.QtWidgets import (
    QApplication, QWidget, QVBoxLayout, QLabel, QLineEdit, QPushButton, QDialog, QGridLayout, QMessageBox, QHBoxLayout
)
from PyQt6.QtGui import QFont, QPixmap, QColor
from PyQt6.QtCore import Qt
import sys
import sympy as sp
import re
import subprocess
import shutil
from io import BytesIO
import numpy as np

import scipy
from sympy import var, lambdify
from sympy import sympify



def fclencurt(N,a,b):   
    n = N-1
    bma = b-a      

    c = np.zeros((N,2)) 

    c[1][1] = 1
    c[0][0] = 2

    for i in range(2,N,2):
        c[i][0] = 2/(1-(i)**2)

    R = np.flipud(c)
    R1 = np.zeros((N-2,2))

    for i in range(0,N-2):
        R1[i] = R[i+1]

    M = np.vstack((c,R1))

    f = np.real(np.fft.ifft(M, axis=0))  


    w = np.ones((N))      
    w[0] = f[0][0]*(bma/2)
    w[N-1] = f[N-1][0]*(bma/2)

    for i in range(1,len(w)-1):
        w[i] = 2*f[i][0]*(bma/2)

    x = np.ones((N))        

    for i in range(0,len(x)):
        x[i] = ((b+a) + n*bma*f[i][1]) *0.5

    w = np.flipud(w)     
    x = np.flipud(x)       
    return [x,w]


def difmatExt(x,N):
    cdif = np.zeros(N)       

    cdif[0] = 2*(-1)**0
    cdif[len(cdif)-1] = 2*(-1)**(len(cdif)-1)

    for i in range(1,len(cdif)-1):
        cdif[i] = 1*(-1)**i

    xdif = np.transpose(np.array([x]))
    Xdif = np.tile(xdif,(1,N))

    dXdif = Xdif - np.transpose(Xdif)

    ccdif = np.transpose(np.array([1/cdif]))   
    cccdif = cdif*ccdif                           
    cccdif = np.transpose(cccdif)

    DL = np.divide(cccdif,(dXdif + np.identity(N)))
    
    sumvec = np.sum(DL, axis=1)

    DL = DL - np.diag(sumvec)
    return DL


def procedura (N, a, b1, b2, delta, beta1, beta2):
    z = a;
    d=1;
    b1 = [str(x) for x in b1]
    b2 = [str(x) for x in b2]
    delta = [str(x) for x in delta]
    beta11=list()
    for s in beta1:
        si = [ str(x) for x in s]
        beta11.append(si)
    beta1=beta11
    beta11=list()
    for s in beta2:
        si = [ str(x) for x in s]
        beta11.append(si)
    beta2=beta11
    N = N+1 
    
    z = sorted(z)           
    l = len(z)
    
    X = np.ones((l-1,N))   
    W = np.ones((l-1,N))    
    Diff = np.zeros((N,N,l-1))  
    
    
    xnodes,w = fclencurt(N, -1, 1)    
    Der = difmatExt(xnodes,N)           
    
    ynodes = (xnodes + 1)*0.5
    
    
    for i in range(l-1):
        X[i] = ynodes*(z[i+1]-z[i]) + z[i]
        W[i] = (w*(z[i+1]-z[i]))*0.5
        Diff[:,:,i] = (2/(z[i+1]-z[i]))*Der
    
    XX = np.reshape(X,N*(l-1))
    
    
    D = Diff[:,:,0]
    
    
    for i in range(1,l-1):
        D = np.vstack((np.hstack(( D, np.zeros( (N+(N-1)*(i-1), N-1) ))),
                       np.hstack(( np.zeros( (N-1, (N-1)*i) ), Diff[1:,:,i]))))
    
    
    DD = D[1:,1:]
    
    II = np.linalg.inv(DD)     
        
    DDD = np.kron(D,np.identity(d))     
    DDDD = np.kron(DD,np.identity(d))   
    IIII = np.kron(II,np.identity(d))   
    
    
    Lb1 = []
    Lb2 = []
    Lbeta1 = []
    Lbeta2 = []
    Ldelta = []
    
    xfun = var('x')
    yfun = var('y')
    
    for i in range(l-1):        
        expr = sympify(b1[i])
        expres = lambdify(xfun, expr)
        Lb1.append(expres)
    
    for i in range(l-1):        
        expr = sympify(b2[i])
        expres = lambdify(xfun, expr)
        Lb2.append(expres)
    
    for i in range(l-1):        
        expr = sympify(delta[i])
        expres = lambdify(xfun, expr)
        Ldelta.append(expres)
    
    for i in range(l-1):        
        li = []
        for j in range(l-1):
            expr = sympify(beta1[i][j])
            expres = lambdify([xfun, yfun], expr)
            li.append(expres)
        Lbeta1.append(li)
    
    for i in range(l-1):        
        li = []
        for j in range(l-1):
            expr = sympify(beta2[i][j])
            expres = lambdify([xfun, yfun], expr)
            li.append(expres)
        Lbeta2.append(li)
    
     
    BB1 = np.zeros(N*(l-1)-(l-2))
    vec = np.zeros(N)
    B1 = np.zeros(0)
    
    for i in range(l-1):
        for j in range(N):
            vec[j] = (Lb1[i](X[i][j]))*W[i][j]
        B1 = np.hstack((B1,vec))
    
    i = 0; j = 0
    
    while i < d*N*(l-1):
        BB1[j] = B1[i]
        t = (i+1) % (N*d)
        if (((t >= d*(N-1)+1)) or (t == 0)) and (i < d*N*(l-2)):
            BB1[j] = BB1[j] + B1[i+d]
        if t == 0:
            i = i+d
        i = i+1; j = j+1
    
    BB1 = np.tile(BB1,(N*(l-1)-(l-2),1))  
    
    BB1 = np.matmul(BB1,DDD)    
    
    BB1 = BB1[d:,d:]
    
    
    
    BETA1 = np.zeros((N*(l-1)-(l-2),N*(l-1)))
    BBETA = np.zeros(0)
    
    h = 0; p = 0; s = 0;
    
    c1 = np.zeros(N)
    
    while h < N*(l-1):
        if ((h+1) % N) == 1 and h != 0:
            h = h+1; s = s+1;
        for i in range(l-1):
            for j in range(N):
                c1[j] = (Lbeta1[s][i] (XX[h], X[i,j]) )*W[i,j]
            BBETA = np.hstack((BBETA,c1))
        BETA1[p] = BBETA
        BBETA = np.zeros(0)
        h = h+1; p = p+1;
    
    BETABETA1 = np.zeros((N*(l-1)-(l-2), N*(l-1)-(l-2)))
    
    h = 0; p = 0;
    
    while h < N*(l-1):
        if ((h+1) % N) == 1 and h != 0:
            h = h+1;
        i = 0; j = 0;
        while i < d*N*(l-1):
            BETABETA1[p,j] = BETA1[p,i]
            t = (i+1) % (N*d)
            if (((t >= d*(N-1)+1)) or (t == 0)) and (i < d*N*(l-2)):
                BETABETA1[p,j] = BETABETA1[p,j] + BETA1[p,i+d]
            if t == 0:
                i = i+d
            i = i+1; j = j+1;
        h = h+1; p = p+1;
    
    
    BETABETA1 = np.matmul(BETABETA1, DDD)
    
    BETABETA1 = BETABETA1[d:,d:]
    
    BETABETA1 = np.matmul(IIII, BETABETA1)
    
    
    BN = BETABETA1 + BB1    
    
    
    
    DELTA = np.zeros(0)
    vec = np.zeros(N)
    DELTADELTA = np.ones(N*(l-1)-(l-2))
    
    for i in range(l-1):
        for j in range(N):
            vec[j] = Ldelta[i](X[i][j])
        DELTA = np.hstack((DELTA,vec))
    
    i = 0; j = 0
    
    while i < d*N*(l-1):
        DELTADELTA[j] = DELTA[i]
        t = (i+1) % N*d
        if t == 0:
            i = i+d
        i = i+1; j = j+1
    
    
    DELTADELTA = np.diag(DELTADELTA)
    DELTADELTA = DELTADELTA[d:,d:]
    DELTADELTA = np.matmul(np.matmul(IIII, DELTADELTA), DDDD)
    
    
    
    BB2 = np.zeros(N*(l-1)-(l-2))
    vec = np.zeros(N)
    B2 = np.zeros(0)
    
    for i in range(l-1):
        for j in range(N):
            vec[j] = (Lb2[i](X[i][j]))*W[i][j]
        B2 = np.hstack((B2,vec))
    
    i = 0; j = 0
    
    while i < d*N*(l-1):
        BB2[j] = B2[i]
        t = (i+1) % (N*d)
        if (((t >= d*(N-1)+1)) or (t == 0)) and (i < d*N*(l-2)):
            BB2[j] = BB2[j] + B2[i+d]
        if t == 0:
            i = i+d
        i = i+1; j = j+1
    
    BB2 = np.tile(BB2,(N*(l-1)-(l-2),1))  
    
    BB2 = np.matmul(BB2,DDD)   
    
    BB2 = BB2[d:,d:]
    
    
    
    BETA2 = np.zeros((N*(l-1)-(l-2),N*(l-1)))
    BBETA = np.zeros(0)
    
    h = 0; p = 0; s = 0;
    
    c2 = np.zeros(N)
    
    while h < N*(l-1):
        if ((h+1) % N) == 1 and h != 0:
            h = h+1; s = s+1;
        for i in range(l-1):
            for j in range(N):
                c2[j] = (Lbeta2[s][i] (XX[h], X[i,j]) )*W[i,j]
            BBETA = np.hstack((BBETA,c2))
        BETA2[p] = BBETA
        BBETA = np.zeros(0)
        h = h+1; p = p+1;
    
    BETABETA2 = np.zeros((N*(l-1)-(l-2), N*(l-1)-(l-2)))
    
    h = 0; p = 0;
    
    while h < N*(l-1):
        if ((h+1) % N) == 1 and h != 0:
            h = h+1;
        i = 0; j = 0;
        while i < d*N*(l-1):
            BETABETA2[p,j] = BETA2[p,i]
            t = (i+1) % (N*d)
            if (((t >= d*(N-1)+1)) or (t == 0)) and (i < d*N*(l-2)):
                BETABETA2[p,j] = BETABETA2[p,j] + BETA2[p,i+d]
            if t == 0:
                i = i+d
            i = i+1; j = j+1;
        h = h+1; p = p+1;
    
    
    BETABETA2 = np.matmul(BETABETA2, DDD)   
    
    BETABETA2 = BETABETA2[d:,d:]
    
    BETABETA2 = np.matmul(IIII, BETABETA2)
    
    MN = DDDD - DELTADELTA - BETABETA2 - BB2    
   
        
    eigenval, eigenvec = scipy.linalg.eig(BN,MN)

    eigenval = np.real(eigenval)

    eigenvec = np.real(eigenvec)

    R0 = max(abs(eigenval))

    eigcomplop = np.real(scipy.linalg.eigvals(BN-MN))  

    r = max((eigcomplop))
    
    eigenvecmax = eigenvec[:,np.argmax(eigenval)]
    eigenvecmax = np.matmul(MN,eigenvecmax)

    index = (np.argmax(np.abs([eigenvecmax])))
    eigenvecmax_norm = (eigenvecmax/eigenvecmax[index])


    F = np.zeros(0)
    DF = np.zeros(0)
    L = np.zeros(0)

    h=1000      # passo

    for i in range(l-1):
        asse_x = []
        asse_x.extend(list(X[i,1:]))
        lin = np.arange(z[i],z[i+1], 1/h)
        eigenf = eigenvecmax_norm[i*(N-1):(i+1)*(N-1)]
        
        psi = scipy.interpolate.CubicSpline(asse_x, eigenf)
        psi1 = psi(lin)
        
        dpsi = scipy.interpolate.PPoly.derivative(psi,nu=1)
        dpsi1 = dpsi(lin)
        
        F = np.hstack((F,psi1))
        DF = np.hstack((DF,dpsi1))
        L = np.hstack((L,lin))
       
    return R0, r, L, F, DF




def is_latex_installed():
    return shutil.which("latex") is not None

if not is_latex_installed():
    QMessageBox.critical(None, "Error", "LaTeX is not installed. Install TeX Live or MikTeX to view formulas.")


class MatrixInputWindow(QDialog):
    def __init__(self, a_length, parent):
        super().__init__(parent)
        self.a_length = (a_length)-1
        self.parent = parent
        self.setWindowTitle("Enter coefficients")
        self.initUI()

    def initUI(self):
        layout = QVBoxLayout()

        title = QLabel("Enter coefficients")
        title.setFont(QFont("Arial", 14, QFont.Weight.Bold))
        layout.addWidget(title, alignment=Qt.AlignmentFlag.AlignCenter)

        self.grid_layout = QGridLayout()
        self.coeff_inputs = {}
        labels = ["b1", "b2", "delta", "beta1", "beta2"]

        for i, label in enumerate(labels[:3]):
            row_inputs = []
            text_label = QLabel(label)
            self.grid_layout.addWidget(text_label, i, 0)

            for j in range(self.a_length):
                cell = QLineEdit()
                cell.setFont(QFont("Arial", 12))
                self.grid_layout.addWidget(cell, i, j + 1)
                row_inputs.append(cell)
            self.coeff_inputs[label] = row_inputs

        for i, label in enumerate(labels[3:]):
            matrix_inputs = []
            text_label = QLabel(label)
            start_row = len(labels[:3]) + i * self.a_length
            self.grid_layout.addWidget(text_label, start_row, 0)

            for j in range(self.a_length):
                row_inputs = []
                for k in range(self.a_length):
                    cell = QLineEdit()
                    cell.setFont(QFont("Arial", 12))
                    self.grid_layout.addWidget(cell, start_row + j, k + 1)
                    row_inputs.append(cell)
                matrix_inputs.append(row_inputs)
            self.coeff_inputs[label] = matrix_inputs

        layout.addLayout(self.grid_layout)

        self.btn_back = QPushButton("Return to main window")
        self.btn_back.setFont(QFont("Arial", 12, QFont.Weight.Bold))
        self.btn_back.clicked.connect(self.closeAndSave)
        layout.addWidget(self.btn_back)

        self.setLayout(layout)

    def closeAndSave(self):
        values = {k: [[cell.text() if cell.text() else '0' for cell in row] for row in v] if isinstance(v[0], list) else [cell.text() if cell.text() else '0' for cell in v] for k, v in self.coeff_inputs.items()}

        data_entered = False
        for key, value in values.items():
            if isinstance(value, list):
                for item in value:
                    if isinstance(item, list):
                        for sub_item in item:
                            if sub_item != '0':
                                data_entered = True
                                break
                        if data_entered:
                            break
                    elif item != '0':
                        data_entered = True
                        break
            if data_entered:
                break

        try:
            b1 = [sp.sympify(x) for x in values['b1']]
            b2 = [sp.sympify(x) for x in values['b2']]
            delta = [sp.sympify(x) for x in values['delta']]
            beta1 = [[sp.sympify(x) for x in row] for row in values['beta1']]
            beta2 = [[sp.sympify(x) for x in row] for row in values['beta2']]

            self.parent.updateLatexDisplay(b1, b2, delta, beta1, beta2)
            self.parent.data_entered = data_entered  
            self.parent.updateDataEnteredDisplay()  
            self.close()
        except (ValueError, SyntaxError) as e:
            print(f"Error in data conversion: {e}")
            QMessageBox.warning(self, "Input error", "Enter valid expressions or valid numbers.")


class LatexWindow(QDialog):
    def __init__(self, latex_vars, parent=None):
        super().__init__(parent)
        self.latex_vars = latex_vars
        self.setWindowTitle("Show LaTeX")
        layout = QVBoxLayout()
        self.image_labels = {}  
        for name, var in self.latex_vars.items():
            image_label = QLabel(self)
            self.image_labels[name] = image_label  

            if var:
                self.update_latex_image(name, var, label=name)  
            image_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            layout.addWidget(image_label)

        close_button = QPushButton("Close", self)
        close_button.setFont(QFont("Arial", 12, QFont.Weight.Bold))
        close_button.clicked.connect(self.accept)  
        layout.addWidget(close_button)

        self.setLayout(layout)

    def update_latex_image(self, name, latex, label=""):
        pixmap = self.render_latex_to_pixmap(latex, label=label)
        if pixmap:
            self.image_labels[name].setPixmap(pixmap)
        else:
            self.image_labels[name].setText(f"Error in LaTeX rendering for {name}")
        self.repaint()

    def render_latex_to_pixmap(self, latex, fontsize=12, label=""):  
        try:
            fig, ax = plt.subplots(figsize=(4, 0.7), dpi=100)
            if label:
                latex_with_label = rf"{label} = {latex}"
            else:
                latex_with_label = latex
            ax.text(0.5, 0.5, f"${latex_with_label}$", fontsize=fontsize, ha='center', va='center')
            ax.axis('off')

            buffer = BytesIO()
            fig.savefig(buffer, format='png', bbox_inches='tight', pad_inches=0.05)
            plt.close(fig)
            buffer.seek(0)

            pixmap = QPixmap()
            if not pixmap.loadFromData(buffer.getvalue()):
                print("Error in loading LaTeX figure")
            return pixmap
        except Exception as e:
            print(f"Error in LaTeX rendering: {e}")
            return None


class MainWindow(QWidget):
    def __init__(self):
        super().__init__()

        self.setWindowTitle("Calculation of reproduction numbers")

        self.data_entered = False

        self.initUI()

        self.a_values = []
        self.b1 = []
        self.b2 = []
        self.delta = []
        self.beta1 = []
        self.beta2 = []
        self.x = sp.symbols('x')
        self.y = sp.symbols('y')

    def initUI(self):
        layout = QVBoxLayout()

        self.title_label = QLabel("Calculation of reproduction numbers")
        self.title_label.setFont(QFont("Arial", 16, QFont.Weight.Bold))
        layout.addWidget(self.title_label, alignment=Qt.AlignmentFlag.AlignCenter)

        latex_formula = r"\begin{cases}\partial_{t}x(t,a)+\partial_{a}x(t,a)=\displaystyle{\int_{0}^{a^{\dagger}}\beta(a,\xi)x(t,\xi)\,d\xi}+\delta(a)x(t,a) \\x(t,0)=\displaystyle{\int_{0}^{a^{\dagger}} b(a)x(t,a)\,da}\end{cases}"
        self.formula_label = QLabel()
        self.set_latex(self.formula_label, latex_formula, tight_layout=True)
        layout.addWidget(self.formula_label, alignment=Qt.AlignmentFlag.AlignCenter)

        self.input_field_a = QLineEdit()
        self.input_field_a.setFont(QFont("Arial", 12))
        layout.addWidget(QLabel("Enter age interval including discontinuity points:"))
        layout.addWidget(self.input_field_a)

        self.btn_open_matrix = QPushButton("Enter coefficients")
        self.btn_open_matrix.setFont(QFont("Arial", 12, QFont.Weight.Bold))
        self.btn_open_matrix.clicked.connect(self.openMatrixInputWindow)
        layout.addWidget(self.btn_open_matrix)

        
        self.data_entered_label = QLabel("Data not entered")
        self.data_entered_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        layout.addWidget(self.data_entered_label)

        self.latex_display_button = QPushButton("Show LaTeX")
        self.latex_display_button.setFont(QFont("Arial", 12, QFont.Weight.Bold))
        self.latex_display_button.clicked.connect(self.showLatexWindow)
        layout.addWidget(self.latex_display_button)

        self.btn_run_function = QPushButton("Calculate")
        self.btn_run_function.setFont(QFont("Arial", 12, QFont.Weight.Bold))
        self.btn_run_function.clicked.connect(self.runFunction)
        layout.addWidget(self.btn_run_function)


        self.plot_label = QLabel()
        self.plot_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        layout.addWidget(self.plot_label)

        self.output_label = QLabel()
        self.output_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.output_label.setFont(QFont("Arial", 14, QFont.Weight.Bold))  
        layout.addWidget(self.output_label)
        
        self.output_label1 = QLabel()
        self.output_label1.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.output_label1.setFont(QFont("Arial", 14, QFont.Weight.Bold))  
        layout.addWidget(self.output_label1)

        self.setLayout(layout)

        self.b1_latex = ""
        self.b2_latex = ""
        self.delta_latex = ""
        self.beta1_latex = ""
        self.beta2_latex = ""

    def set_latex(self, label, latex_code, fontsize=16, tight_layout=False):
        from PyQt6.QtGui import QPixmap
        try:
            fig, ax = plt.subplots(tight_layout=tight_layout, figsize=(4, 1))
            ax.text(0.5, 0.5, r"$" + latex_code + r"$", fontsize=fontsize, ha='center', va='center')
            ax.axis('off')

            buffer = BytesIO()
            plt.savefig(buffer, format='png', bbox_inches='tight', dpi=100)
            plt.close(fig)  
            buffer.seek(0)

            q_image = QPixmap()
            q_image.loadFromData(buffer.getvalue())

            label.setPixmap(q_image)
            label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        except Exception as e:
            print(f"Error in LaTeX rendering: {e}")
            label.setText(f"Error: {e}")
        finally:
            plt.close()

    def openMatrixInputWindow(self):
        try:
            a_str = self.input_field_a.text()
            self.a_values = [sp.sympify(x) for x in eval(a_str)] 

            if isinstance(self.a_values, list):
                self.matrixWindow = MatrixInputWindow(len(self.a_values), self)
                self.matrixWindow.exec()
            else:
                print("Error: enter a valid list of numbers for 'a'!")
                QMessageBox.warning(self, "Input error", "Enter a valid list of numbers for 'a'!")
        except Exception as e:
            print("Error: enter a valid list of numbers for 'a'!", e)
            QMessageBox.warning(self, "Input error", "Enter a valid list of numbers for 'a'!")

    def updateLatexDisplay(self, b1, b2, delta, beta1, beta2):
        def remove_brackets(latex_string):
            latex_string = re.sub(r"\\left\[", "", latex_string)
            latex_string = re.sub(r"\\right\]", "", latex_string)
            latex_string = re.sub(r"\\begin{matrix}", "", latex_string)  
            latex_string = re.sub(r"\\end{matrix}", "", latex_string)    
            return latex_string

        b1_matrix = sp.Matrix(b1)
        b2_matrix = sp.Matrix(b2)
        delta_matrix = sp.Matrix(delta)
        beta1_matrix = sp.Matrix(beta1)
        beta2_matrix = sp.Matrix(beta2)

        self.b1_latex = r"\begin{bmatrix} " + remove_brackets(sp.latex(b1_matrix)) + r" \end{bmatrix}"
        self.b2_latex = r"\begin{bmatrix} " + remove_brackets(sp.latex(b2_matrix)) + r" \end{bmatrix}"
        self.delta_latex = r"\begin{bmatrix} " + remove_brackets(sp.latex(delta_matrix)) + r" \end{bmatrix}"
        self.beta1_latex = r"\begin{bmatrix} " + remove_brackets(sp.latex(beta1_matrix)) + r" \end{bmatrix}"
        self.beta2_latex = r"\begin{bmatrix} " + remove_brackets(sp.latex(beta2_matrix)) + r" \end{bmatrix}"

        self.b1 = b1
        self.b2 = b2
        self.delta = delta  
        self.beta1 = beta1  
        self.beta2 = beta2  


        if hasattr(self, 'latex_window') and self.latex_window.isVisible():
            self.latex_window.update_latex_image("b1", self.b1_latex, label="b1")
            self.latex_window.update_latex_image("b2", self.b2_latex, label="b2")
            self.latex_window.update_latex_image("delta", self.delta_latex, label="delta")
            self.latex_window.update_latex_image("beta1", self.beta1_latex, label="beta1")
            self.latex_window.update_latex_image("beta2", self.beta2_latex, label="beta2")

    def showLatexWindow(self):

        self.latex_window = LatexWindow(
            {
                "b^{+}": self.b1_latex,
                "b^{-}": self.b2_latex,
                "\delta": self.delta_latex,
                "\\beta^{+}": self.beta1_latex,
                "\\beta^{-}": self.beta2_latex,
            },
            self,
        )
        self.latex_window.exec()

    def updateDataEnteredDisplay(self):
        """Aggiorna il testo e il colore dell'etichetta data_entered_label."""
        if self.data_entered:
            self.data_entered_label.setText("Data entered correctly!")
            self.data_entered_label.setStyleSheet("color: green;") 
        else:
            self.data_entered_label.setText("Data not entered")
            self.data_entered_label.setStyleSheet("color: red;")  

    def runFunction(self):
        if not self.data_entered:
            QMessageBox.warning(self, "Warning", "Enter data before computing.")
            return

    
        R0, r, L, F, DF = self.my_function(self.a_values, self.b1, self.b2, self.delta, self.beta1, self.beta2)

       
        try:
            if hasattr(R0, 'evalf'): 
                 R0_numeric = float(R0.evalf())
            else:
                R0_numeric = float(R0) 
            self.output_label.setText(f"Reproduction number R: {R0_numeric:.10f}")  

        except Exception as e:
            self.output_label.setText(f"Error during the calculation of R: {e}")
            
        try:
            if hasattr(r, 'evalf'):
                 r_numeric = float(r.evalf())
            else:
                r_numeric = float(r) 
            self.output_label1.setText(f"Malthusian parameter r: {r_numeric:.10f}")

        except Exception as e:
            self.output_label1.setText(f"Error during the calculation of r: {e}")


        self.generate_plot(L, F, DF)

    def my_function(self, a, b1, b2, delta, beta1, beta2):
        
        R0, r, L, F, DF = procedura(20, a, b1, b2, delta, beta1, beta2)

        return [R0, r, L, F, DF]

    def generate_plot(self, L, F, DF):
        try:
            
            fig, axes = plt.subplots(nrows =1, ncols=2, figsize=(8, 4), dpi=50)
            axes[0].plot(L,F)
            axes[0].set_title("Cumulative distribution")

            axes[1].plot(L,DF, color='red')
            axes[1].plot(0,F[0],'o') 
            axes[1].set_title("Dominant eigenfunction")
          
           
            buffer = BytesIO()
            fig.savefig(buffer, format='png', bbox_inches='tight')
            plt.close(fig)
            buffer.seek(0)

            pixmap = QPixmap()
            if not pixmap.loadFromData(buffer.getvalue()):
                print("Error in loading plot")
                return

            self.plot_label.setPixmap(pixmap)
        except Exception as e:
            print(f"Error in plotting: {e}")
            self.plot_label.setText(f"Error in plotting: {e}")
        return 0

if __name__ == "__main__":
    app = QApplication(sys.argv)
    mainWin = MainWindow()
    mainWin.show()
    sys.exit(app.exec())